//LinkList 1.9 FINAL by Jason Winnebeck 5/25/99
template <class linkdata>
//Private function used in getting items
Link<linkdata> *LinkList<linkdata>::GetIndx(int indx) {
  Link<linkdata> *temp = first; //temp pointer to cycle with
  for (int c=0; c < indx; c++) {//Cycle through until we hit
    temp = temp->next;
  }
  return temp;
}

template <class linkdata>
LinkList<linkdata>::LinkList() {
  first = NULL;          //There are no elements yet
  len = 0;               //""
  SetToStart();          //set the next pointer to the start
}

template <class linkdata>
LinkList<linkdata>::~LinkList() {
  int l = len;
  for (int c=0; c < l; c++)   //delete all the elements
    DeleteItem(0);
}

template <class linkdata>
void LinkList<linkdata>::Clear() {
  int l = len;
  for (int c=0; c < l; c++)
    DeleteItem(0);
  SetToStart();
}

template <class linkdata>
void LinkList<linkdata>::AddItem(const linkdata &data) {
  Link<linkdata> *newitem = new Link<linkdata>;//new Link
  newitem->data = new linkdata;//new data that the link will contain
  *newitem->data = data;       //COPY the data over
  newitem->next = first;       //link it
  first = newitem;             //set the first link
  len++;                       //increase the list's length
}

template <class linkdata>
void LinkList<linkdata>::AddItem(linkdata *data) {
  Link<linkdata> *newitem = new Link<linkdata>;//new link
  newitem->data = new linkdata;//new data that the link will contain
  *newitem->data = *data;      //COPY the data over
  newitem->next = first;       //link it
  first = newitem;             //set the first link
  len++;                       //increase the list's length
}

template <class linkdata>
bool LinkList<linkdata>::DeleteItem(int indx) {
  if ((indx < len) && (indx >= 0)) {               //Range Check
    Link<linkdata> *prev = NULL;  //we want to get the previous link, so trace
    Link<linkdata> *temp = first; //will become the link we will delete
    if (indx) {                   //If not element 0
      for (int c=0; c < indx; c++) {//Search for the indx
        prev = temp;
        temp = temp->next;
      }
      prev->next = temp->next;    //"Skip" over this element
    } else {                      //Element 0 requires special delete
      temp = first;               // Basically inverse of AddItem()
      first = temp->next;         // skips over the 1st element
    }
    delete temp->data;            //delete the old memory
    delete temp;
    len--;                        //Reduce list's length
    return true;                  //Delete successful
  }
  return false;                   //Element does not exist
}

template <class linkdata>
void LinkList<linkdata>::Swap(LinkList<linkdata> &l) {
  Link<linkdata> *first2, *next2, *prev2;
  int len2;
  //Need member-by-member manual copy so as to not call destructor/constructors
  first2 = first;
  next2 = next;
  prev2 = prev;
  len2 = len;
  first = l.first;
  next = l.next;
  prev = l.prev;
  len = l.len;
  l.first = first2;
  l.next = next2;
  l.prev = prev2;
  l.len = len2;
}

template <class linkdata>
linkdata *LinkList<linkdata>::GetItem(int indx) {
  if ((indx < len) && (indx >= 0)) {               //Range Checking
    linkdata *copy = new linkdata;//We need a new space so we can COPY data
    *copy = *GetIndx(indx)->data; //COPY data over from the link we want
    return copy;                  //return pointer to the data
  }
  return NULL;                    //return "error"
}

template <class linkdata>
linkdata& LinkList<linkdata>::operator [] (int indx) {
  if ((indx < len) && (indx >= 0)) {
    return *GetIndx(indx)->data;  //return reference to data
  }
  return *GetIndx(len-1)->data;   //I HAVE to return SOMETHING, and this is
                                  //the best I can come up with!!!
}

template <class linkdata>
bool GetItem(int indx, linkdata& dat) {//GetItem for non-ptr
  if ((indx < len) && (indx >= 0)) {                //Range Checking
    dat = *GetIndx(indx)->data;    //copy over dat from the indx we need
    return true;                   //True for successful
  }
  return false;                    //False if not successful
}

template <class linkdata>
void LinkList<linkdata>::SetItem(int indx, const linkdata& dat) {
  if ((indx < len) && (indx >= 0)) {                //Range check
    *GetIndx(indx)->data = dat;    //Get the link we need and copy data over
  }
  //On failure, don't set ANYTHING
}

template <class linkdata>
linkdata *LinkList<linkdata>::MakeLinear() {
  if (len > 0) {
    linkdata *ret = new linkdata[len];
    SetToStart();
    for (int c=0; c<len; c++) {
      ret[c] = *_GetNext();
      _Advance();
    }
    return ret;
  } else
    return NULL;  //No data here!
}

template <class linkdata>
bool LinkList<linkdata>::GetNext(linkdata& dat) {
  if (next == NULL) {      //let's not run off the array
    return false;
  } else {
    dat = *next->data;     //copy data back
    next = next->next;     //set pointer to next's pointer to the next element
    return true;
  }
}

//*** ADVANCED FUNCTIONS ***
template <class linkdata>
void LinkList<linkdata>::_MoveData(linkdata *dat) {
  Link<linkdata> *newitem = new Link<linkdata>;
  newitem->data = dat;  //just copy the pointer, not the data
  newitem->next = first;//Just like AddItem . . .
  first = newitem;
  len++;
}

template <class linkdata>
void LinkList<linkdata>::_AddHere(linkdata *dat) {
  if (prev == NULL) {
    AddItem(dat); //If at the beginning, do a normal add
  } else {
    Link<linkdata> *newitem = new Link<linkdata>; //new link
    newitem->data = new linkdata;//new data
    *newitem->data = *dat;       //Copy data over
    newitem->next = next;        //Similar to AddHere. . .
    prev->next = newitem;        //Insert item
    prev = newitem;              //We're the element before now
    len++;                       //increase length
  }
}

template <class linkdata>
void LinkList<linkdata>::_MoveHere(linkdata *dat) {
  if (prev == NULL) {
    _MoveData(dat);
  } else {
    Link<linkdata> *newitem = new Link<linkdata>;
    newitem->data = dat;
    newitem->next = next;
    prev->next = newitem;
    prev = newitem;
    len++;
  }
}

template <class linkdata>
void LinkList<linkdata>::_DeleteThis() {
  if (prev != NULL)          //if not at the beginning
    prev->next = next->next; //"Skip" this element to delete
  else                       //if we are at the beginning
    first = next->next;
  Link<linkdata> *temp = next; //Temp var to delete after data is used
  next = next->next;           //advance, or set next to end (NULL)
  delete temp->data;           //delete the data
  delete temp;                 //delete the link
  len--;                       //decrement length
}
