//Below is the 78 chars line!
//----------------------------------------------------------------------------
//Jason Winnebeck

#ifndef _SERVER_H_
#define _SERVER_H_

#include <arpa/inet.h>  //for sockadd_in !!!
#include "linklist.h"
#include <time.h>

#include "muudpic.h"    //Global constants
#include "window.h"

const int MAXCLI = 16;
const int TIMEOUT = 10; //Seconds before client drop

extern int gin;         //Total data received
extern int gout;        //Total data sent

extern int bsock;       //Broadcasting socket

struct Clidata {
  char* name;           //Stores client's name
  sockaddr_in loc;      //Stores location data
  clock_t start, end;   //Used in measuring ping times.
  int pingtime;         //Clients current ping time

  Clidata() : name(NULL), start(0), end(0), pingtime(0) {}
  ~Clidata() {if (name) delete[] name;}
};

extern LinkList<Clidata> Clients;

extern char buf[512];    //Global recieve buffer
const  int BUFLEN = 511; //Length of buffer

extern CWindow Stats;
extern CWindow Message;
extern CWindow Clilist;

//Global stuff
void DisplayError();     //atexit() errno display
void UpdateStats();      //updates connection stats window
void UpdateClilist();    //updates Client list
int  Init();             //Inits globals, returns ERROR on error

//Basic Networking Functions
void SendMsg(Clidata* cli, char* msg);
                           //Formats msg with name header and CRLF footer
                           //Then transmits to clients
void SendOut(Clidata* cli, char* msg);
                           //Sends the entire string to a client
void RecvFrom(Clidata* &cli);
                           //Recieves an entire message (NULL separated)
                           //cli will become the client who sent buf
                           //sends data to global buffer
void Broadcast(char* msg); //Sends message to all clients
void BroadcastPing();      //Sends pings to all clients

//Server Functions
int  NewCli(int sock);     //Returns ERROR if the client is "invalid"
int  SetupListen();        //puts the socket on listen mode
int  SetupBroadcast();     //creates the actual UDP chat socket
void SendNewCli(Clidata* cli);              //Sends clients join notice
void LeaveCli(Clidata* cli);                //Sends clients exit notice
void SendPing();                            //Pings all clients

#endif
